clear
clc
load F14nominal
G=zpk(F14nominal);
A_S = [tf([25 0],[1 25]); tf(25,[1 25])];
A_R = A_S;
freq = 12.5 * (2*pi);  % 12.5 Hz
zeta = 0.5;
antia_filt_yaw = tf(freq^2,[1 2*zeta*freq freq^2]);
antia_filt_lat = tf(freq^2,[1 2*zeta*freq freq^2]);
 
freq = 4.1 * (2*pi);  % 4.1 Hz
zeta = 0.7;
antia_filt_roll = tf(freq^2,[1 2*zeta*freq freq^2]);

antia_filt = append(antia_filt_roll,antia_filt_yaw,antia_filt_lat);

W_n = append(0.025,tf(0.0125*[1 1],[1 100]),0.025);

clf
figure(1)
bodemag(W_n(2,2))
title('Sensor noise power as a function of frequency')

figure(2)
hq_p    = 5.0 * tf(2.0,[1 2.0]);
step(hq_p)
title('Desired response from lateral stick to roll rate (Handling Quality)')

figure(3)
hq_beta = -2.5 * tf(1.25^2,[1 2.5 1.25^2]);
step(hq_beta)
title('Desired response from rudder pedal to side-slip angle (Handling Quality)')

W_act = diag([1/90,1/20,1/125,1/30]);
W_p = tf([0.05 2.9 105.93 6.17 0.16],[1 9.19 30.80 18.83 3.95]);
WP_1 = tf([1 9.19 30.80 18.83 3.95],[0.05 2.9 105.93 6.17 0.16]);

figure(4)
bodemag(WP_1)
title('Weight on Handling Quality spec')

W_beta = 2*W_p;
% Normalized error dynamics
Delta_G = ultidyn('Delta_G',[2 2],'Bound',1.0);
 
% Frequency shaping of error dynamics
w_1 = tf(2.0*[1 4],[1 160]);
w_2 = tf(1.5*[1 20],[1 200]);
W_in = append(w_1,w_2);
figure(5)
bodemag(w_1,'-',w_2,'--')
title('Relative error on nominal F14 model as a function of frequency')
legend('stabilizer','rudder','Location','NorthWest');

systemnames = 'F14nominal A_S A_R W_in Delta_G';
inputvar = '[delta_dstab; delta_rud]';
outputvar = '[A_S; A_R; F14nominal]';
input_to_F14nominal = '[A_S(2); A_R(2)]';
input_to_A_S = '[delta_dstab + W_in(1)]';
input_to_A_R = '[delta_rud + W_in(2)]';
input_to_W_in = '[Delta_G]';
input_to_Delta_G = '[delta_dstab; delta_rud]';
sysoutname = 'F14_unc';
cleanupsysic = 'yes';
sysic;

% Pick 10 random samples
F14_unc_sampl = usample(F14_unc,10);
 
% Look at response from differential stabilizer to beta
figure(6)
subplot(211), step(F14_unc.Nominal(5,1),'r+',F14_unc_sampl(5,1),'b:',10)
legend('Nominal','Perturbed'),ylabel('\beta (degrees)')
 
subplot(212), bodemag(F14_unc.Nominal(5,1),'r+',F14_unc_sampl(5,1),'b:',{0.001,1e3})
legend('Nominal','Perturbed')

systemnames = 'F14_unc antia_filt hq_p hq_beta ';
systemnames = [systemnames ' W_act W_n W_p W_beta'];
inputvar  = '[sn_nois{3}; roll_cmd; beta_cmd; delta_dstab; delta_rud]';
outputvar = '[ W_p; W_beta; W_act;  roll_cmd; beta_cmd; antia_filt + W_n ]';
input_to_F14_unc       = '[ delta_dstab; delta_rud ]';
input_to_antia_filt = '[ F14_unc(6:8) ]';
input_to_hq_beta = '[ beta_cmd ]';
input_to_hq_p    = '[ roll_cmd ]';
input_to_W_act      = '[ F14_unc(1:4) ]';
input_to_W_beta     = '[ hq_beta - F14_unc(5) ]';
input_to_W_p        = '[ hq_p - F14_unc(6) ]';
input_to_W_n        = '[ sn_nois ]';
sysoutname = 'F14IC';
cleanupsysic = 'yes';
sysic

inputvar  = '[sn_nois{3}; roll_cmd; beta_cmd; delta_dstab; delta_rud]';
nmeas = 5;      % number of measurements
nctrls = 2;     % number of controls
[kinf,ginf,gammainf] = hinfsyn(F14IC.NominalValue,nmeas,nctrls);
figure(7)
omega=logspace(-1,5,500);
sig_g1=sigma(ginf,omega);
semilogx(omega,sig_g1(1,:))

fmu = logspace(-2,4,200);
opt = dksynOptions('FrequencyVector',fmu,'NumberofAutoIterations',5, 'Display','On');
[kmu,clpmu,bnd] = dksyn(F14IC,nmeas,nctrls,opt);

clinf = lft(F14IC,kinf);
clmu = lft(F14IC,kmu);

clinfg = ufrd(clinf,fmu);
clmug = ufrd(clmu,fmu);

opt = wcgainOptions('MaxOverFrequency','off');  % options for WCGAIN
 
% Compute worst-case gain (as a function of frequency) for kinf
[mginf,wcuinf] = wcgain(clinfg,opt);

% Compute worst-case gain for kinf
[mgmu,wcumu,infomu] = wcgain(clmug,opt);

figure(8)
subplot(211)
semilogx(fnorm(clinfg.NominalValue),'r',mginf.UpperBound,'-.b');
title('Performance analysis for K_\infty')
xlabel('Frequency (rad/sec)')
ylabel('Closed-loop gain');
legend('Nominal Plant','Worst-Case','Location','NorthWest');
 
subplot(212)
semilogx(fnorm(clmug.NominalValue),'r',mgmu.UpperBound,'-.b');
title('Performance analysis for K_\mu')
xlabel('Frequency (rad/sec)')
ylabel('Closed-loop gain');
legend('Nominal Plant','Worst-Case','Location','SouthWest');


systemnames = 'F14_unc antia_filt kmu';
inputvar  = '[roll_cmd; beta_cmd]';
outputvar = '[F14_unc(6); F14_unc(5)]';
input_to_F14_unc       = '[ kmu ]';
input_to_antia_filt = '[ F14_unc(6:8) ]';
input_to_kmu = '[ roll_cmd; beta_cmd; antia_filt ]';
sysoutname = 'F14SIM';
cleanupsysic = 'yes';
sysic

time = 0:0.02:15;
u_stick = (time>=9 & time<12);
u_pedal = (time>=1 & time<4) - (time>=4 & time<7);
 
figure(9)
subplot(211), plot(time,u_stick), axis([0 14 -2 2]), title('Lateral stick command')
subplot(212), plot(time,u_pedal), axis([0 14 -2 2]), title('Rudder pedal command')


% Ideal behavior
F14ideal = append(hq_p,hq_beta);
 
% Worst-case response
F14wc = usubs(F14SIM,wcumu);
 
% Simulate and compare responses
figure(10)
lsim(F14ideal,'g',F14SIM.NominalValue,'r',F14wc,'b--',[u_stick ; u_pedal],time)
legend('ideal','nominal','perturbed','Location','SouthEast');
title('Closed-loop responses with mu controller K_\mu')


